const Promise = require('bluebird');
const fs = require('fs');
const path = require('path');
const dateTime = require('node-datetime');
const zipdir = Promise.promisify(require('zip-dir'));
const deploy = require('./util/deploy');
const logger = require('./util/logger');

const dt = dateTime.create();
const formattedDate = dt.format('Y-m-d-H-M-S');

const omittedDefault = /(config\/((?!admin\.properties\.js).)*$)|node_modules$|^.*\.(scss|map|less)$|\.git$|__tests__$|build$|src$|package*$|yarn.lock$|\.circle$|bash[.]*|gulp[.]*|docker[.]*|deploy[.]*|deployment[.]*|pagebuilder[.]*|mongo[.]*|proxy[.]*/g;

/**
* @desc Bundles and prepares the PageBuilder assets so it can be posted to the API endpoint. Creates a .zip file.
* @param {string} rootDirectory - A string describing the location of the generted .zip file.
* @param {string} endpoint - A string containing the API endpoint the .zip file should deploy to.
* @param {string} username - A string containing the API username.
* @param {string} password - A string containing the API password.
* @param {string} omittedContent - An optional regex expression that correlates to the content that should be omitted from the bundle zip.
* @param {string} environment - A string containing the environment destination of the deployment.
* @param {function} postDeploy - An optional function that will be called when the deployment has finished.
**/
module.exports = ({
  rootDirectory,
  endpoint,
  username,
  password,
  omittedContent = omittedDefault,
  environment,
  postDeploy = null,
}) => (() => {
  logger.bundleInitialize();
  const bundleDirectory = path.join(rootDirectory, '/deployment/');
  const bundleFile = `${bundleDirectory}${formattedDate}.zip`;

  // Creates the bundle directory if it doesn't exist.
  if (!fs.existsSync(bundleDirectory)) {
    fs.mkdirSync(bundleDirectory);
  }

  const options = {
    filter: (path, stat) => !(omittedContent.test(path)),
    saveTo: bundleFile,
    each: path => logger.zipEach(path)
  };

  const cb = (err, buffer) => {
    if (err) {
      logger.bundleError(err);
    } else {
      logger.bundleSuccess(bundleFile);
      deploy({
        environment,
        endpoint,
        username,
        password,
        postDeploy,
        path: bundleFile,
        name: formattedDate
      });
    }
  };
  return setTimeout(() => {
    zipdir(rootDirectory, options, cb);
  }, 2000);
});

